# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - ApproveChanges

    Permit the approval of an edited page queued by the page editor when an
    unprivileged user attempted to save a page. Since queued changes are placed
    in a subpage, this action just moves the queued page content into the
    existing page when approving the changes.

    @copyright: 2011 Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

Dependencies = ['pages']

from MoinMoin.action import ActionBase
from MoinMoin.Page import Page
from MoinMoin.PageEditor import PageEditor
from ApproveChangesSupport import *

# Action class and supporting functions.

class ApproveChanges(ActionBase):

    "An action which approves a queued page version."

    def __init__(self, pagename, request):
        ActionBase.__init__(self, pagename, request)
        _ = self._
        self.form_trigger = "approve"
        self.form_trigger_label = _("Approve changes")

    def get_form_html(self, buttons_html):
        _ = self._
        request = self.request
        fmt = request.formatter
        form = get_form(request)
        rev = form.get("rev")

        d = {
            "buttons_html"          : buttons_html,
            "prompt"                : escape(_("Approve the displayed page version?")),
            }

        # Prepare the output HTML.

        html = '''
<table>
    <tr>
        <td>%(prompt)s</td>
    </tr>
    <tr>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>''' % d

        if rev:
            html += '''
<input name="rev" type="hidden" value="%(rev)s" />''' % {"rev" : escattr(rev[0])}

        return html

    def do_action(self):

        "Approve the page and move it into place."

        _ = self._
        request = self.request

        # Make sure that only suitably privileged users can perform this action.

        queued_changes_page = get_queued_changes_page(request)

        if not is_reviewer(request):
            return 0, _("Only page reviewers can perform this action.")

        # Edit the target page, using this page's content.
        # The current page must be a queued page version.

        if not is_queued_changes_page(request, self.pagename):
            return 0, _("This page is not queued for approval.")

        target_page_name = get_target_page_name(self.pagename)
        target_page = PageEditor(request, target_page_name)

        # First, the displayed revision must be retrieved.

        form = get_form(request)

        # Get the revision or None.

        rev = form.get("rev")
        if rev is None:
            rev = self.page.current_rev()
        else:
            rev = int(rev[0])

        page = Page(request, self.page.page_name, rev=rev)
        body = page.get_raw_body()

        # Save the target page, removing any introduced directives.

        body = remove_directives(body, ["acl", "parent-revision"])

        try:
            target_page.saveText(body, 0, comment=_("Changes to page approved from queue revision %d.") % rev)
        except PageEditor.Unchanged:
            pass

        # Delete the current page.
        # NOTE: The page could be deleted completely or certain revisions
        # NOTE: purged.
        # NOTE: (to-do/proper-queued-page-deletion.txt)

        current_page = PageEditor(request, self.pagename)
        current_page.deletePage(_("Changes to page approved."))

        # Redirect to the target page.

        request.http_redirect(target_page.url(request))
        return 1, None

    def render_success(self, msg, msgtype):

        """
        Render neither 'msg' nor 'msgtype' since redirection should occur
        instead.
        NOTE: msgtype is optional because MoinMoin 1.5.x does not support it.
        """

        pass

# Action function.

def execute(pagename, request):
    ApproveChanges(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
